@@ -66,12 +66,6 @@ const int rpmFLAGS = RPMSENSE_EQUAL;
 #undef HASHTYPE
 #undef HTKEYTYPE
 
-enum addOp_e {
-    RPMTE_INSTALL	= 0,
-    RPMTE_UPGRADE	= 1,
-    RPMTE_REINSTALL	= 2,
-};
-
 /**
  * Check for supported payload format in header.
  * @param h		header to check
@@ -126,7 +120,7 @@ static int removePackage(rpmts ts, Header h, rpmte depends)
 	return 0;
     }
 
-    p = rpmteNew(ts, h, TR_REMOVED, NULL, NULL);
+    p = rpmteNew(ts, h, TR_REMOVED, NULL, NULL, 0);
     if (p == NULL)
 	return 1;
 
@@ -178,23 +172,22 @@ static int addSelfErasures(rpmts ts, rpm_color_t tscolor, int op,
     Header oh;
     rpmdbMatchIterator mi = rpmtsInitIterator(ts, RPMDBI_NAME, rpmteN(p), 0);
     int rc = 0;
-    int cmp;
 
     while ((oh = rpmdbNextIterator(mi)) != NULL) {
 	/* Ignore colored packages not in our rainbow. */
 	if (skipColor(tscolor, hcolor, headerGetNumber(oh, RPMTAG_HEADERCOLOR)))
 	    continue;
 
-	cmp = rpmVersionCompare(h, oh);
+	/* On reinstall, skip packages with differing NEVRA. */
+	if (op != RPMTE_UPGRADE) {
+	    char * ohNEVRA = headerGetAsString(oh, RPMTAG_NEVRA);
+	    if (!rstreq(rpmteNEVRA(p), ohNEVRA)) {
+		free(ohNEVRA);
+		continue;
+	    }
+	    free(ohNEVRA);
+	}
 
-	/* On upgrade, skip packages that contain identical NEVR. */
-	if ((op == RPMTE_UPGRADE) && (cmp == 0))
-	    continue;
-
-	/* On reinstall, skip packages with differing NEVR. */
-	if ((op == RPMTE_REINSTALL) && (cmp != 0))
-	    continue;
-	
 	if (removePackage(ts, oh, p)) {
 	    rc = 1;
 	    break;
@@ -222,17 +215,8 @@ static int addObsoleteErasures(rpmts ts, rpm_color_t tscolor, rpmte p)
 	mi = rpmtsPrunedIterator(ts, RPMDBI_NAME, Name, 1);
 
 	while ((oh = rpmdbNextIterator(mi)) != NULL) {
-	    const char *oarch = headerGetString(oh, RPMTAG_ARCH);
 	    int match;
 
-	    /* avoid self-obsoleting packages */
-	    if (rstreq(rpmteN(p), Name) && rstreq(rpmteA(p), oarch)) {
-		char * ohNEVRA = headerGetAsString(oh, RPMTAG_NEVRA);
-		rpmlog(RPMLOG_DEBUG, "  Not obsoleting: %s\n", ohNEVRA);
-		free(ohNEVRA);
-		continue;
-	    }
-
 	    /*
 	     * Rpm prior to 3.0.3 does not have versioned obsoletes.
 	     * If no obsoletes version info is available, match all names.
@@ -438,7 +422,7 @@ static int addPackage(rpmts ts, Header h,
 	    goto exit;
     }
 
-    p = rpmteNew(ts, h, TR_ADDED, key, relocs);
+    p = rpmteNew(ts, h, TR_ADDED, key, relocs, op);
     if (p == NULL) {
 	ec = 1;
 	goto exit;
@@ -833,16 +817,27 @@ static void checkDS(rpmts ts, depCache dcache, rpmte te,
 
 /* Check a given dependency against installed packages */
 static void checkInstDeps(rpmts ts, depCache dcache, rpmte te,
-			  rpmTag depTag, const char *dep)
+			  rpmTag depTag, const char *dep, rpmds depds, int neg)
 {
     Header h;
-    rpmdbMatchIterator mi = rpmtsPrunedIterator(ts, depTag, dep, 1);
+    rpmdbMatchIterator mi;
     rpmstrPool pool = rpmtsPool(ts);
+    char *ndep = NULL;
     /* require-problems are unsatisfied, others appear "satisfied" */
     int is_problem = (depTag == RPMTAG_REQUIRENAME);
 
+    if (depds)
+	dep = rpmdsN(depds);
+    if (neg) {
+	ndep = rmalloc(strlen(dep) + 2);
+	ndep[0] = '!';
+	strcpy(ndep + 1, dep);
+	dep = ndep;
+    }
+
+    mi = rpmtsPrunedIterator(ts, depTag, dep, 1);
     while ((h = rpmdbNextIterator(mi)) != NULL) {
-	char * pkgNEVRA;
+	int match = 1;
 	rpmds ds;
 
 	/* Ignore self-obsoletes and self-conflicts */
@@ -852,26 +847,22 @@ static void checkInstDeps(rpmts ts, depCache dcache, rpmte te,
 		continue;
 	}
 
-	pkgNEVRA = headerGetAsString(h, RPMTAG_NEVRA);
 	ds = rpmdsNewPool(pool, h, depTag, 0);
 	rpmdsSetIx(ds, rpmdbGetIteratorFileNum(mi));
 
-	if (unsatisfiedDepend(ts, dcache, ds) == is_problem)
+	/* Is it in our range at all? (but file deps have no range) */
+	if (depds)
+	    match = rpmdsCompare(ds, depds);
+
+	if (match && unsatisfiedDepend(ts, dcache, ds) == is_problem) {
+	    char *pkgNEVRA = headerGetAsString(h, RPMTAG_NEVRA);
 	    rpmteAddDepProblem(te, pkgNEVRA, ds, NULL);
+	    free(pkgNEVRA);
+	}
 
 	rpmdsFree(ds);
-	free(pkgNEVRA);
     }
     rpmdbFreeIterator(mi);
-}
-
-static void checkNotInstDeps(rpmts ts, depCache dcache, rpmte te,
-			     rpmTag depTag, const char *dep)
-{
-    char *ndep = rmalloc(strlen(dep) + 2);
-    ndep[0] = '!';
-    strcpy(ndep + 1, dep);
-    checkInstDeps(ts, dcache, te, depTag, ndep);
     free(ndep);
 }
 
@@ -907,10 +898,7 @@ static void checkInstFileDeps(rpmts ts, depCache dcache, rpmte te,
 	} else {
 	    continue;
 	}
-	if (!is_not)
-	    checkInstDeps(ts, dcache, te, depTag, dep);
-	else
-	    checkNotInstDeps(ts, dcache, te, depTag, dep);
+	checkInstDeps(ts, dcache, te, depTag, dep, NULL, is_not);
 	_free(fpdep);
     }
     _free(fp);
@@ -1063,10 +1051,9 @@ int rpmtsCheck(rpmts ts)
 
 	/* Check provides against conflicts in installed packages. */
 	while (rpmdsNext(provides) >= 0) {
-	    const char *dep = rpmdsN(provides);
-	    checkInstDeps(ts, dcache, p, RPMTAG_CONFLICTNAME, dep);
-	    if (reqnothash && depexistsHashHasEntry(reqnothash, dep))
-		checkNotInstDeps(ts, dcache, p, RPMTAG_REQUIRENAME, dep);
+	    checkInstDeps(ts, dcache, p, RPMTAG_CONFLICTNAME, NULL, provides, 0);
+	    if (reqnothash && depexistsHashHasEntry(reqnothash, rpmdsN(provides)))
+		checkInstDeps(ts, dcache, p, RPMTAG_REQUIRENAME, NULL, provides, 1);
 	}
 
 	/* Skip obsoletion checks for source packages (ie build) */
@@ -1074,7 +1061,7 @@ int rpmtsCheck(rpmts ts)
 	    continue;
 
 	/* Check package name (not provides!) against installed obsoletes */
-	checkInstDeps(ts, dcache, p, RPMTAG_OBSOLETENAME, rpmteN(p));
+	checkInstDeps(ts, dcache, p, RPMTAG_OBSOLETENAME, NULL, rpmteDS(p, RPMTAG_NAME), 0);
 
 	/* Check filenames against installed conflicts */
         if (confilehash || reqnotfilehash) {
@@ -1104,10 +1091,9 @@ int rpmtsCheck(rpmts ts)
 
 	/* Check provides and filenames against installed dependencies. */
 	while (rpmdsNext(provides) >= 0) {
-	    const char *dep = rpmdsN(provides);
-	    checkInstDeps(ts, dcache, p, RPMTAG_REQUIRENAME, dep);
-	    if (connothash && depexistsHashHasEntry(connothash, dep))
-		checkNotInstDeps(ts, dcache, p, RPMTAG_CONFLICTNAME, dep);
+	    checkInstDeps(ts, dcache, p, RPMTAG_REQUIRENAME, NULL, provides, 0);
+	    if (connothash && depexistsHashHasEntry(connothash, rpmdsN(provides)))
+		checkInstDeps(ts, dcache, p, RPMTAG_CONFLICTNAME, NULL, provides, 1);
 	}
 
 	if (reqfilehash || connotfilehash) {
