@@ -251,10 +251,7 @@ static Header headerCreate(void *blob, int32_t indexLen)
     }
     h->instance = 0;
     h->sorted = HEADERSORT_NONE;
-
-    h->index = (h->indexAlloced
-	? xcalloc(h->indexAlloced, sizeof(*h->index))
-	: NULL);
+    h->index = xcalloc(h->indexAlloced, sizeof(*h->index));
 
     h->nrefs = 0;
     return headerLink(h);
@@ -963,8 +960,7 @@ rpmRC hdrblobImport(hdrblob blob, int fast, Header *hdrp, char **emsg)
     h->sorted = HEADERSORT_NONE;
     headerSort(h);
     h->flags |= HEADERFLAG_ALLOCATED;
-    if (hdrp)
-	*hdrp = h;
+    *hdrp = h;
 
     /* We own the memory now, avoid double-frees */
     blob->ei = NULL;
@@ -1910,7 +1906,7 @@ rpmRC hdrblobRead(FD_t fd, int magic, int exact_size, rpmTagVal regionTag, hdrbl
 
     if (regionTag == RPMTAG_HEADERSIGNATURES) {
 	il_max = 32;
-	dl_max = 8192;
+	dl_max = 64 * 1024;
     }
 
     memset(block, 0, sizeof(block));
