@@ -65,6 +65,12 @@ void headerMergeLegacySigs(Header h, Header sigh)
 	case RPMSIGTAG_PAYLOADSIZE:
 	    td.tag = RPMTAG_ARCHIVESIZE;
 	    break;
+	case RPMSIGTAG_FILESIGNATURES:
+	    td.tag = RPMTAG_FILESIGNATURES;
+	    break;
+	case RPMSIGTAG_FILESIGNATURELENGTH:
+	    td.tag = RPMTAG_FILESIGNATURELENGTH;
+	    break;
 	case RPMSIGTAG_SHA1:
 	case RPMSIGTAG_SHA256:
 	case RPMSIGTAG_DSA:
@@ -88,11 +94,11 @@ void headerMergeLegacySigs(Header h, Header sigh)
 		    continue;
 		break;
 	    case RPM_STRING_TYPE:
+	    case RPM_STRING_ARRAY_TYPE:
 	    case RPM_BIN_TYPE:
 		if (td.count >= 16*1024)
 		    continue;
 		break;
-	    case RPM_STRING_ARRAY_TYPE:
 	    case RPM_I18NSTRING_TYPE:
 		continue;
 		break;
