@@ -83,7 +83,7 @@ static void printFileInfo(const char * name,
 	(void)strftime(timefield, sizeof(timefield) - 1, fmt, tm);
     }
 
-    rpmlog(RPMLOG_NOTICE, "%s %4d %-8s%-8s %10s %s %s\n", perms,
+    rpmlog(RPMLOG_NOTICE, "%s %4d %-8s %-8s %10s %s %s\n", perms,
 	(int)nlink, ownerfield, groupfield, sizefield, timefield, 
 	link ? link : name);
     free(perms);
@@ -109,6 +109,10 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	}
     }
 
+    /* Inclusion flags traditionally imply list mode */
+    if (qva->qva_incattr)
+	qva->qva_flags |= QUERY_FOR_LIST;
+
     if (!(qva->qva_flags & QUERY_FOR_LIST))
 	goto exit;
 
@@ -135,24 +139,12 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	const char *flink = rpmfiFLink(fi);
 	char *buf = NULL;
 
-	/* If querying only docs, skip non-doc files. */
-	if ((qva->qva_flags & QUERY_FOR_DOCS) && !(fflags & RPMFILE_DOC))
-	    continue;
-
-	/* If querying only configs, skip non-config files. */
-	if ((qva->qva_flags & QUERY_FOR_CONFIG) && !(fflags & RPMFILE_CONFIG))
-	    continue;
-
-	/* If querying only licenses, skip non-license files. */
-	if ((qva->qva_flags & QUERY_FOR_LICENSE) && !(fflags & RPMFILE_LICENSE))
-	    continue;
-
-	/* If querying only ... yes we know the drill, and this is dumb. */
-	if ((qva->qva_flags & QUERY_FOR_ARTIFACT) && !(fflags & RPMFILE_ARTIFACT))
+	/* If filtering by inclusion, skip non-matching (eg --configfiles) */
+	if (qva->qva_incattr && !(fflags & qva->qva_incattr))
 	    continue;
 
 	/* Skip on attributes (eg from --noghost) */
-	if (fflags & qva->qva_fflags)
+	if (fflags & qva->qva_excattr)
 	    continue;
 
 	if (qva->qva_flags & QUERY_FOR_STATE) {
@@ -500,8 +492,7 @@ static rpmdbMatchIterator initQueryIterator(QVA_t qva, rpmts ts, const char * ar
 	}
 	mi = rpmdbFreeIterator(mi);
 	if (! matches) {
-	    size_t l = strlen(arg);
-	    if (!(l > 4 && !strcmp(arg + l - 4, ".rpm")))
+	    if (!rpmFileHasSuffix(arg, ".rpm"))
 		rpmlog(RPMLOG_NOTICE, _("package %s is not installed\n"), arg);
 	} else {
 	    mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
@@ -580,8 +571,7 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_const_t argv)
 	    rpmdbMatchIterator mi = initQueryIterator(qva, ts, *arg);
 	    ecLocal = rpmcliShowMatches(qva, ts, mi);
 	    if (mi == NULL && qva->qva_source == RPMQV_PACKAGE) {
-		size_t l = strlen(*arg);
-		if (l > 4 && !strcmp(*arg + l - 4, ".rpm")) {
+		if (rpmFileHasSuffix(*arg, ".rpm")) {
 		    char * const argFirst[2] = { arg[0], NULL };
 		    rpmgi gi = rpmgiNew(ts, giFlags, argFirst);
 		    ecLocal = rpmgiShowMatches(qva, ts, gi);
@@ -606,7 +596,8 @@ int rpmcliQuery(rpmts ts, QVA_t qva, char * const * argv)
 	qva->qva_showPackage = showQueryPackage;
 
     /* If --queryformat unspecified, then set default now. */
-    if (!(qva->qva_flags & _QUERY_FOR_BITS) && qva->qva_queryFormat == NULL) {
+    if (!(qva->qva_flags & _QUERY_FOR_BITS) && !(qva->qva_incattr) &&
+					    qva->qva_queryFormat == NULL) {
 	char * fmt = rpmExpand("%{?_query_all_fmt}\n", NULL);
 	if (fmt == NULL || strlen(fmt) <= 1) {
 	    free(fmt);
@@ -616,12 +607,6 @@ int rpmcliQuery(rpmts ts, QVA_t qva, char * const * argv)
     }
 
     vsflags = rpmExpandNumeric("%{?_vsflags_query}");
-    if (rpmcliQueryFlags & VERIFY_DIGEST)
-	vsflags |= _RPMVSF_NODIGESTS;
-    if (rpmcliQueryFlags & VERIFY_SIGNATURE)
-	vsflags |= _RPMVSF_NOSIGNATURES;
-    if (rpmcliQueryFlags & VERIFY_HDRCHK)
-	vsflags |= RPMVSF_NOHDRCHK;
     vsflags |= rpmcliVSFlags;
 
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
