@@ -24,8 +24,6 @@
 
 #include "debug.h"
 
-int _print_pkts = 0;
-
 static int doImport(rpmts ts, const char *fn, char *buf, ssize_t blen)
 {
     char const * const pgpmark = "-----BEGIN PGP ";
@@ -187,11 +185,11 @@ rpmRC rpmpkgRead(struct rpmvs_s *vs, FD_t fd,
     /* Finalize header range */
     rpmvsFiniRange(vs, RPMSIG_HEADER);
 
-    /* Unless disabled, read the payload, generating digest(s) on the fly. */
-    if (!(rpmvsFlags(vs) & RPMVSF_NEEDPAYLOAD)) {
-	/* Fish interesting tags from the main header. This is a bit hacky... */
-	rpmvsAppendTag(vs, blob, RPMTAG_PAYLOADDIGEST);
+    /* Fish interesting tags from the main header. This is a bit hacky... */
+    rpmvsAppendTag(vs, blob, RPMTAG_PAYLOADDIGEST);
 
+    /* If needed and not explicitly disabled, read the payload as well. */
+    if (rpmvsRange(vs) & RPMSIG_PAYLOAD) {
 	/* Initialize digests ranging over the payload only */
 	rpmvsInitRange(vs, RPMSIG_PAYLOAD);
 
@@ -283,10 +281,6 @@ int rpmcliVerifySignatures(rpmts ts, ARGV_const_t argv)
     rpmVSFlags vsflags = rpmtsVfyFlags(ts);
     int vfylevel = rpmtsVfyLevel(ts);
 
-    if (rpmcliQueryFlags & QUERY_DIGEST)
-	vsflags |= _RPMVSF_NODIGESTS;
-    if (rpmcliQueryFlags & QUERY_SIGNATURE)
-	vsflags |= _RPMVSF_NOSIGNATURES;
     vsflags |= rpmcliVSFlags;
     if (rpmcliVfyLevelMask) {
 	vfylevel &= ~rpmcliVfyLevelMask;
