@@ -1588,6 +1588,7 @@ int rpmdbExtendIterator(rpmdbMatchIterator mi,
 	    dbiIndexSetAppendSet(mi->mi_set, set, 0);
 	    dbiIndexSetFree(set);
 	}
+	mi->mi_sorted = 0;
 	rc = 0;
     }
 
@@ -1649,6 +1650,7 @@ int rpmdbAppendIterator(rpmdbMatchIterator mi,
 
     for (unsigned int i = 0; i < nHdrNums; i++)
 	dbiIndexSetAppendOne(mi->mi_set, hdrNums[i], 0, 0);
+    mi->mi_sorted = 0;
     return 0;
 }
 
@@ -1973,7 +1975,7 @@ unsigned int rpmdbIndexIteratorPkgOffset(rpmdbIndexIterator ii, unsigned int nr)
     return dbiIndexRecordOffset(ii->ii_set, nr);
 }
 
-unsigned int *rpmdbIndexIteratorPkgOffsets(rpmdbIndexIterator ii)
+const unsigned int *rpmdbIndexIteratorPkgOffsets(rpmdbIndexIterator ii)
 {
     int i;
 
@@ -2565,7 +2567,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 	    /* Deleted entries are eliminated in legacy headers by copy. */
 	    if (headerIsEntry(h, RPMTAG_HEADERIMAGE)) {
 		Header nh = headerReload(headerCopy(h), RPMTAG_HEADERIMAGE);
-		rc = rpmdbAdd(newdb, nh);
+		rc = rpmdbAdd(newdb, h);
 		headerFree(nh);
 	    } else {
 		rc = rpmdbAdd(newdb, h);
@@ -2644,3 +2646,23 @@ int rpmdbCtrl(rpmdb db, rpmdbCtrlOp ctrl)
     return dbctrl ? dbCtrl(db, dbctrl) : 1;
 }
 
+char *rpmdbCookie(rpmdb db)
+{
+    void *cookie = NULL;
+    rpmdbIndexIterator ii = rpmdbIndexIteratorInit(db, RPMDBI_NAME);
+
+    if (ii) {
+	DIGEST_CTX ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
+	const void *key = 0;
+	size_t keylen = 0;
+	while ((rpmdbIndexIteratorNext(ii, &key, &keylen)) == 0) {
+	    const unsigned int *offsets = rpmdbIndexIteratorPkgOffsets(ii);
+	    unsigned int npkgs = rpmdbIndexIteratorNumPkgs(ii);
+	    rpmDigestUpdate(ctx, key, keylen);
+	    rpmDigestUpdate(ctx, offsets, sizeof(*offsets) * npkgs);
+	}
+	rpmDigestFinal(ctx, &cookie, NULL, 1);
+    }
+    rpmdbIndexIteratorFree(ii);
+    return cookie;
+}
