@@ -781,28 +781,17 @@ int rpmfilesStat(rpmfiles fi, int ix, int flags, struct stat *sb)
 	int warn = flags & 0x1;
 	const char *user = rpmfilesFUser(fi, ix);
 	const char *group = rpmfilesFGroup(fi, ix);
-	const int * hardlinks = NULL;
-	uint32_t nlinks = rpmfilesFLinks(fi, ix, &hardlinks);
 
 	memset(sb, 0, sizeof(*sb));
-	sb->st_nlink = nlinks;
+	sb->st_nlink = rpmfilesFLinks(fi, ix, NULL);
 	sb->st_ino = rpmfilesFInode(fi, ix);
 	sb->st_rdev = rpmfilesFRdev(fi, ix);
 	sb->st_mode = rpmfilesFMode(fi, ix);
 	sb->st_mtime = rpmfilesFMtime(fi, ix);
 
 	/* Only regular files and symlinks have a size */
-	if (S_ISREG(sb->st_mode)) {
-	    /* Content and thus size comes with last hardlink */
-	    if (!(nlinks > 1 && hardlinks[nlinks-1] != ix))
-		sb->st_size = rpmfilesFSize(fi, ix);
-	} else if (S_ISLNK(sb->st_mode)) {
-	    /*
-	     * Normally rpmfilesFSize() is correct for symlinks too, this is
-	     * only needed for glob()'ed links from fakechroot environment.
-	     */
-	    sb->st_size = strlen(rpmfilesFLink(fi, ix));
-	}
+	if (S_ISREG(sb->st_mode) || S_ISLNK(sb->st_mode))
+	    sb->st_size = rpmfilesFSize(fi, ix);
 
 	if (user && rpmugUid(user, &sb->st_uid)) {
 	    if (warn)
@@ -1004,7 +993,7 @@ int rpmfileContentsEqual(rpmfiles ofi, int oix, rpmfiles nfi, int nix)
 	    goto exit;
 	}
 
-	if (rpmDoDigest(nalgo, fn, 0, (unsigned char *)buffer, NULL) != 0) {
+	if (rpmDoDigest(nalgo, fn, 0, (unsigned char *)buffer) != 0) {
 	     goto exit;		/* assume file has been removed */
 	}
 
@@ -1088,7 +1077,7 @@ rpmFileAction rpmfilesDecideFate(rpmfiles ofi, int oix,
 	/* See if the file on disk is identical to the one in new pkg */
 	ndigest = rpmfilesFDigest(nfi, nix, &nalgo, &ndiglen);
 	if (diskWhat == REG && newWhat == REG) {
-	    if (rpmDoDigest(nalgo, fn, 0, (unsigned char *)buffer, NULL))
+	    if (rpmDoDigest(nalgo, fn, 0, (unsigned char *)buffer))
 		goto exit;		/* assume file has been removed */
 	    if (ndigest && memcmp(ndigest, buffer, ndiglen) == 0) {
 		action = FA_TOUCH;
@@ -1101,7 +1090,7 @@ rpmFileAction rpmfilesDecideFate(rpmfiles ofi, int oix,
 	if (diskWhat == REG) {
 	    /* hash algo changed or digest was not computed, recalculate it */
 	    if ((oalgo != nalgo) || (newWhat != REG)) {
-		if (rpmDoDigest(oalgo, fn, 0, (unsigned char *)buffer, NULL))
+		if (rpmDoDigest(oalgo, fn, 0, (unsigned char *)buffer))
 		    goto exit;	/* assume file has been removed */
 		}
 	    if (odigest && memcmp(odigest, buffer, odiglen) == 0)
@@ -1227,7 +1216,7 @@ int rpmfilesConfigConflict(rpmfiles fi, int ix)
 	int algo;
 	size_t diglen;
 	const unsigned char *ndigest = rpmfilesFDigest(fi,ix, &algo, &diglen);
-	if (rpmDoDigest(algo, fn, 0, (unsigned char *)buffer, NULL))
+	if (rpmDoDigest(algo, fn, 0, (unsigned char *)buffer))
 	    goto exit;	/* assume file has been removed */
 	if (ndigest && memcmp(ndigest, buffer, diglen) == 0)
 	    goto exit;	/* unmodified config file */
@@ -1888,7 +1877,17 @@ uint32_t rpmfiFDepends(rpmfi fi, const uint32_t ** fddictp)
 
 int rpmfiStat(rpmfi fi, int flags, struct stat *sb)
 {
-    return rpmfilesStat(fi->files, fi->i, flags, sb);
+    int rc = -1;
+    if (fi != NULL) {
+	rc = rpmfilesStat(fi->files, fi->i, flags, sb);
+	/* In archives, hardlinked files are empty except for the last one */
+	if (rc == 0 && fi->archive && sb->st_nlink > 1) {
+	    const int *links = NULL;
+	    if (rpmfiFLinks(fi, &links) && links[sb->st_nlink-1] != fi->i)
+		sb->st_size = 0;
+	}
+    }
+    return rc;
 }
 
 int rpmfiCompare(const rpmfi afi, const rpmfi bfi)
