@@ -188,7 +188,7 @@ int runPostUnTransFileTrigs(rpmts ts)
 	headerGet(trigH, RPMTAG_INSTPREFIXES, &installPrefixes,
 		HEADERGET_ALLOC|HEADERGET_ARGV);
 
-	nerrors += runScript(ts, NULL, trigH, installPrefixes.data, script, 0, 0);
+	nerrors += runScript(ts, NULL, trigH, installPrefixes.data, script, 0, -1);
 	rpmtdFreeData(&installPrefixes);
 	rpmScriptFree(script);
 	headerFree(trigH);
@@ -437,8 +437,8 @@ static int runHandleTriggersInPkg(rpmts ts, rpmte te, Header h,
 	    inputFunc = (char *(*)(void *)) matchFilesNext;
 	    rpmScriptSetNextFileFunc(script, inputFunc, mfi);
 
-	    nerrors += runScript(ts, te, h, installPrefixes.data,
-				script, 0, 0);
+	    nerrors += runScript(ts, NULL, h, installPrefixes.data,
+				script, 0, -1);
 	    rpmtdFreeData(&installPrefixes);
 	    rpmScriptFree(script);
 	}
