@@ -12,6 +12,11 @@
 
 static struct selabel_handle * sehandle = NULL;
 
+static inline rpmlogLvl loglvl(int iserror)
+{
+    return iserror ? RPMLOG_ERR : RPMLOG_DEBUG;
+}
+
 static void sehandle_fini(int close_status)
 {
     if (sehandle) {
@@ -47,7 +52,7 @@ static rpmRC sehandle_init(int open_status)
 
     sehandle = selabel_open(SELABEL_CTX_FILE, opts, 1);
 
-    rpmlog(RPMLOG_DEBUG, "selabel_open: (%s) %s\n",
+    rpmlog(loglvl(sehandle == NULL), "selabel_open: (%s) %s\n",
 	   path, (sehandle == NULL ? strerror(errno) : ""));
 
     return (sehandle != NULL) ? RPMRC_OK : RPMRC_FAIL;
@@ -89,63 +94,65 @@ static rpmRC selinux_psm_pre(rpmPlugin plugin, rpmte te)
     return rc;
 }
 
-#ifndef HAVE_SETEXECFILECON
-static int setexecfilecon(const char *path, const char *fallback_type)
+static rpmRC selinux_scriptlet_fork_post(rpmPlugin plugin,
+						 const char *path, int type)
 {
-    int rc = -1;
+    rpmRC rc = RPMRC_FAIL;
+    int xx;
+#ifndef HAVE_SETEXECFILECON
     security_context_t mycon = NULL, fcon = NULL, newcon = NULL;
     context_t con = NULL;
 
+    if (sehandle == NULL)
+	return RPMRC_OK;
+
     /* Figure the context to for next exec() */
     if (getcon(&mycon) < 0)
 	goto exit;
     if (getfilecon(path, &fcon) < 0)
 	goto exit;
-    if (security_compute_create(mycon, fcon,
-			string_to_security_class("process"), &newcon) < 0)
+    if (security_compute_create(mycon, fcon, string_to_security_class("process"), &newcon) < 0)
 	goto exit;
 
     if (rstreq(mycon, newcon)) {
+	/* No default transition, use rpm_script_t for now. */
+	const char * script_type = "rpm_script_t";
+
 	con = context_new(mycon);
 	if (!con)
 	    goto exit;
-	if (context_type_set(con, fallback_type))
+	if (context_type_set(con, script_type))
 	    goto exit;
 	freecon(newcon);
 	newcon = xstrdup(context_str(con));
     }
 
-    rc = setexeccon(newcon);
+    if ((xx = setexeccon(newcon)) == 0)
+	rc = RPMRC_OK;
+
+    rpmlog(loglvl(xx < 0), "setexeccon: (%s, %s) %s\n",
+	       path, newcon, (xx < 0 ? strerror(errno) : ""));
 
 exit:
     context_free(con);
     freecon(newcon);
     freecon(fcon);
     freecon(mycon);
-    return rc;
-}
-#endif
-
-static rpmRC selinux_scriptlet_fork_post(rpmPlugin plugin,
-						 const char *path, int type)
-{
-    /* No default transition, use rpm_script_t for now. */
-    const char *script_type  = "rpm_script_t";
-    rpmRC rc = RPMRC_FAIL;
 
+#else
     if (sehandle == NULL)
 	return RPMRC_OK;
 
-    if (setexecfilecon(path, script_type) == 0)
+    if ((xx = setexecfilecon(path, "rpm_script_t")) == 0)
 	rc = RPMRC_OK;
 
+    rpmlog(loglvl(xx < 0), "setexecfilecon: (%s) %s\n",
+	       path, (xx < 0 ? strerror(errno) : ""));
+#endif
     /* If selinux is not enforcing, we don't care either */
     if (rc && security_getenforce() < 1)
 	rc = RPMRC_OK;
 
-    rpmlog(rc ? RPMLOG_ERR : RPMLOG_DEBUG, "setexecfilecon: (%s, %s) %s\n",
-	   path, script_type, rc ? strerror(errno) : "");
-
     return rc;
 }
 
@@ -161,13 +168,11 @@ static rpmRC selinux_fsm_file_prepare(rpmPlugin plugin, rpmfi fi,
 	if (selabel_lookup_raw(sehandle, &scon, dest, file_mode) == 0) {
 	    int conrc = lsetfilecon(path, scon);
 
+	    rpmlog(loglvl(conrc < 0), "lsetfilecon: (%s, %s) %s\n",
+		       path, scon, (conrc < 0 ? strerror(errno) : ""));
+
 	    if (conrc == 0 || (conrc < 0 && errno == EOPNOTSUPP))
 		rc = RPMRC_OK;
-
-	    rpmlog((rc != RPMRC_OK) ? RPMLOG_ERR : RPMLOG_DEBUG,
-		   "lsetfilecon: (%s, %s) %s\n",
-		   path, scon, (conrc < 0 ? strerror(errno) : ""));
-
 	    freecon(scon);
 	} else {
 	    /* No context for dest is not our headache */
