@@ -6,6 +6,7 @@
 #include <rpm/rpmsq.h>
 #include <rpm/rpmlog.h>
 #include <rpm/rpmmacro.h>
+#include <rpm/rpmsign.h>
 
 #include "header-py.h"
 #include "rpmarchive-py.h"
@@ -22,6 +23,7 @@
 #include "rpmtd-py.h"
 #include "rpmte-py.h"
 #include "rpmts-py.h"
+#include "spec-py.h"
 
 /** \ingroup python
  * \name Module: rpm
@@ -156,6 +158,38 @@ static PyObject * setInterruptSafety(PyObject * self, PyObject * args, PyObject
     Py_RETURN_NONE;
 }
 
+static int parseSignArgs(PyObject * args, PyObject *kwds,
+			const char **path, struct rpmSignArgs *sargs)
+{
+    char * kwlist[] = { "path", "keyid", "hashalgo", NULL };
+
+    memset(sargs, 0, sizeof(*sargs));
+    return PyArg_ParseTupleAndKeywords(args, kwds, "s|si", kwlist,
+				    path, &sargs->keyid, &sargs->hashalgo);
+}
+
+static PyObject * addSign(PyObject * self, PyObject * args, PyObject *kwds)
+{
+    const char *path = NULL;
+    struct rpmSignArgs sargs;
+
+    if (!parseSignArgs(args, kwds, &path, &sargs))
+	return NULL;
+
+    return PyBool_FromLong(rpmPkgSign(path, &sargs) == 0);
+}
+
+static PyObject * delSign(PyObject * self, PyObject * args, PyObject *kwds)
+{
+    const char *path = NULL;
+    struct rpmSignArgs sargs;
+
+    if (!parseSignArgs(args, kwds, &path, &sargs))
+	return NULL;
+
+    return PyBool_FromLong(rpmPkgDelSign(path, &sargs) == 0);
+}
+
 static PyMethodDef rpmModuleMethods[] = {
     { "addMacro", (PyCFunction) rpmmacro_AddMacro, METH_VARARGS|METH_KEYWORDS,
       "rpmPushMacro(macro, value)\n"
@@ -215,6 +249,8 @@ static PyMethodDef rpmModuleMethods[] = {
       "once only at process startup because currently signal handlers will\n"
       "not be retroactively applied if a database is open."
     },
+    { "addSign", (PyCFunction) addSign, METH_VARARGS|METH_KEYWORDS, NULL },
+    { "delSign", (PyCFunction) delSign, METH_VARARGS|METH_KEYWORDS, NULL },
     { NULL }
 } ;
 
@@ -237,7 +273,7 @@ static void addRpmTags(PyObject *module)
 
 	PyModule_AddIntConstant(module, tagname, tagval);
 	pyval = PyInt_FromLong(tagval);
-	pyname = Py_BuildValue("s", shortname);
+	pyname = utf8FromString(shortname);
 	PyDict_SetItem(dict, pyval, pyname);
 	Py_DECREF(pyval);
 	Py_DECREF(pyname);
@@ -264,11 +300,10 @@ static int prepareInitModule(void)
     if (PyType_Ready(&rpmProblem_Type) < 0) return 0;
     if (PyType_Ready(&rpmPubkey_Type) < 0) return 0;
     if (PyType_Ready(&rpmstrPool_Type) < 0) return 0;
-#if 0
-    if (PyType_Ready(&rpmtd_Type) < 0) return 0;
-#endif
     if (PyType_Ready(&rpmte_Type) < 0) return 0;
     if (PyType_Ready(&rpmts_Type) < 0) return 0;
+    if (PyType_Ready(&spec_Type) < 0) return 0;
+    if (PyType_Ready(&specPkg_Type) < 0) return 0;
 
     return 1;
 }
@@ -377,17 +412,17 @@ static int initModule(PyObject *m)
     Py_INCREF(&rpmstrPool_Type);
     PyModule_AddObject(m, "strpool", (PyObject *) &rpmstrPool_Type);
 
-#if 0
-    Py_INCREF(&rpmtd_Type);
-    PyModule_AddObject(m, "td", (PyObject *) &rpmtd_Type);
-#endif
-
     Py_INCREF(&rpmte_Type);
     PyModule_AddObject(m, "te", (PyObject *) &rpmte_Type);
 
     Py_INCREF(&rpmts_Type);
     PyModule_AddObject(m, "ts", (PyObject *) &rpmts_Type);
 
+    Py_INCREF(&spec_Type);
+    PyModule_AddObject(m, "spec", (PyObject *) &spec_Type);
+    Py_INCREF(&specPkg_Type);
+    PyModule_AddObject(m, "specPkg", (PyObject *) &specPkg_Type);
+
     addRpmTags(m);
 
     PyModule_AddStringConstant(m, "__version__", RPMVERSION);
@@ -489,6 +524,7 @@ static int initModule(PyObject *m)
     REGISTER_ENUM(RPMPROB_FILTER_OLDPACKAGE);
     REGISTER_ENUM(RPMPROB_FILTER_DISKSPACE);
     REGISTER_ENUM(RPMPROB_FILTER_DISKNODES);
+    REGISTER_ENUM(RPMPROB_FILTER_VERIFY);
 
     REGISTER_ENUM(RPMCALLBACK_UNKNOWN);
     REGISTER_ENUM(RPMCALLBACK_INST_PROGRESS);
@@ -511,6 +547,9 @@ static int initModule(PyObject *m)
     REGISTER_ENUM(RPMCALLBACK_SCRIPT_STOP);
     REGISTER_ENUM(RPMCALLBACK_INST_STOP);
     REGISTER_ENUM(RPMCALLBACK_ELEM_PROGRESS);
+    REGISTER_ENUM(RPMCALLBACK_VERIFY_PROGRESS);
+    REGISTER_ENUM(RPMCALLBACK_VERIFY_START);
+    REGISTER_ENUM(RPMCALLBACK_VERIFY_STOP);
 
     REGISTER_ENUM(RPMPROB_BADARCH);
     REGISTER_ENUM(RPMPROB_BADOS);
@@ -524,9 +563,7 @@ static int initModule(PyObject *m)
     REGISTER_ENUM(RPMPROB_DISKSPACE);
     REGISTER_ENUM(RPMPROB_DISKNODES);
     REGISTER_ENUM(RPMPROB_OBSOLETES);
-
-    REGISTER_ENUM(VERIFY_DIGEST);
-    REGISTER_ENUM(VERIFY_SIGNATURE);
+    REGISTER_ENUM(RPMPROB_VERIFY);
 
     REGISTER_ENUM(RPMLOG_EMERG);
     REGISTER_ENUM(RPMLOG_ALERT);
@@ -557,9 +594,20 @@ static int initModule(PyObject *m)
     REGISTER_ENUM(_RPMVSF_NOSIGNATURES);
     REGISTER_ENUM(_RPMVSF_NOHEADER);
     REGISTER_ENUM(_RPMVSF_NOPAYLOAD);
+    REGISTER_ENUM(RPMVSF_MASK_NODIGESTS);
+    REGISTER_ENUM(RPMVSF_MASK_NOSIGNATURES);
+    REGISTER_ENUM(RPMVSF_MASK_NOHEADER);
+    REGISTER_ENUM(RPMVSF_MASK_NOPAYLOAD);
+
+    REGISTER_ENUM(RPMSIG_NONE_TYPE);
+    REGISTER_ENUM(RPMSIG_DIGEST_TYPE);
+    REGISTER_ENUM(RPMSIG_SIGNATURE_TYPE);
+    REGISTER_ENUM(RPMSIG_VERIFIABLE_TYPE);
+    REGISTER_ENUM(RPMSIG_UNVERIFIED_TYPE);
 
     REGISTER_ENUM(TR_ADDED);
     REGISTER_ENUM(TR_REMOVED);
+    REGISTER_ENUM(TR_RPMDB);
 
     REGISTER_ENUM(RPMDBI_PACKAGES);
     REGISTER_ENUM(RPMDBI_LABEL);
@@ -595,6 +643,32 @@ static int initModule(PyObject *m)
     REGISTER_ENUM(RPMVERIFY_READFAIL);
     REGISTER_ENUM(RPMVERIFY_LSTATFAIL);
 
+    REGISTER_ENUM(RPMBUILD_ISSOURCE);
+    REGISTER_ENUM(RPMBUILD_ISPATCH);
+    REGISTER_ENUM(RPMBUILD_ISICON);
+    REGISTER_ENUM(RPMBUILD_ISNO);
+
+    REGISTER_ENUM(RPMBUILD_NONE);
+    REGISTER_ENUM(RPMBUILD_PREP);
+    REGISTER_ENUM(RPMBUILD_BUILD);
+    REGISTER_ENUM(RPMBUILD_INSTALL);
+    REGISTER_ENUM(RPMBUILD_CHECK);
+    REGISTER_ENUM(RPMBUILD_CLEAN);
+    REGISTER_ENUM(RPMBUILD_FILECHECK);
+    REGISTER_ENUM(RPMBUILD_PACKAGESOURCE);
+    REGISTER_ENUM(RPMBUILD_PACKAGEBINARY);
+    REGISTER_ENUM(RPMBUILD_RMSOURCE);
+    REGISTER_ENUM(RPMBUILD_RMBUILD);
+    REGISTER_ENUM(RPMBUILD_RMSPEC);
+
+    REGISTER_ENUM(RPMBUILD_PKG_NONE);
+    REGISTER_ENUM(RPMBUILD_PKG_NODIRTOKENS);
+
+    REGISTER_ENUM(RPMSPEC_NONE);
+    REGISTER_ENUM(RPMSPEC_ANYARCH);
+    REGISTER_ENUM(RPMSPEC_FORCE);
+    REGISTER_ENUM(RPMSPEC_NOLANG);
+
     return 1;
 }
 
