@@ -12,6 +12,9 @@
 extern char *optarg;
 extern int optind;
 #endif
+#if HAVE_SCHED_GETAFFINITY
+#include <sched.h>
+#endif
 
 #if !defined(isblank)
 #define	isblank(_c)	((_c) == ' ' || (_c) == '\t')
@@ -119,13 +122,31 @@ static int print_macro_trace = _PRINT_MACRO_TRACE;
 #define	_PRINT_EXPAND_TRACE	0
 static int print_expand_trace = _PRINT_EXPAND_TRACE;
 
+typedef void (*macroFunc)(MacroBuf mb, int chkexist, int negate,
+			const char * f, size_t fn, const char * g, size_t gn);
+typedef const char *(*parseFunc)(MacroBuf mb, const char * se, size_t slen);
+
 /* forward ref */
 static int expandMacro(MacroBuf mb, const char *src, size_t slen);
 static void pushMacro(rpmMacroContext mc,
 	const char * n, const char * o, const char * b, int level, int flags);
 static void popMacro(rpmMacroContext mc, const char * n);
 static int loadMacroFile(rpmMacroContext mc, const char * fn);
+static void doFoo(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn);
+static void doLoad(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn);
+static void doLua(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn);
+static void doOutput(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn);
+static void doTrace(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn);
 
+static const char * doDef(MacroBuf mb, const char * se, size_t slen);
+static const char * doGlobal(MacroBuf mb, const char * se, size_t slen);
+static const char * doDump(MacroBuf mb, const char * se, size_t slen);
+static const char * doUndefine(MacroBuf mb, const char * se, size_t slen);
 /* =============================================================== */
 
 static rpmMacroContext rpmmctxAcquire(rpmMacroContext mc)
@@ -192,15 +213,16 @@ findEntry(rpmMacroContext mc, const char *name, size_t namelen, size_t *pos)
  * @param buf		input buffer
  * @param size		inbut buffer size (bytes)
  * @param f		file handle
- * @return		buffer, or NULL on end-of-file
+ * @return		number of lines read, or 0 on end-of-file
  */
-static char *
+static int
 rdcl(char * buf, size_t size, FILE *f)
 {
     char *q = buf - 1;		/* initialize just before buffer. */
     size_t nb = 0;
     size_t nread = 0;
     int pc = 0, bc = 0;
+    int nlines = 0;
     char *p = buf;
 
     if (f != NULL)
@@ -208,6 +230,7 @@ rdcl(char * buf, size_t size, FILE *f)
 	*(++q) = '\0';			/* terminate and move forward. */
 	if (fgets(q, size, f) == NULL)	/* read next line. */
 	    break;
+	nlines++;
 	nb = strlen(q);
 	nread += nb;			/* trim trailing \r and \n */
 	for (q += nb - 1; nb > 0 && iseol(*q); q--)
@@ -237,12 +260,12 @@ rdcl(char * buf, size_t size, FILE *f)
 	    *(++q) = '\0';		/* trim trailing \r, \n */
 	    break;
 	}
-	q++; p++; nb++;			/* copy newline too */
+	q++; nb++;			/* copy newline too */
 	size -= nb;
 	if (*q == '\r')			/* XXX avoid \r madness */
 	    *q = '\n';
     } while (size > 0);
-    return (nread > 0 ? buf : NULL);
+    return nlines;
 }
 
 /**
@@ -271,6 +294,31 @@ matchchar(const char * p, char pl, char pr)
     return (const char *)NULL;
 }
 
+static void mbErr(MacroBuf mb, int error, const char *fmt, ...)
+{
+    char *emsg = NULL;
+    int n;
+    va_list ap;
+
+    va_start(ap, fmt);
+    n = rvasprintf(&emsg, fmt, ap);
+    va_end(ap);
+
+    if (n >= -1) {
+	/* XXX should have an non-locking version for this */
+	char *pfx = rpmExpand("%{?__file_name:%{__file_name}: }",
+			      "%{?__file_lineno:line %{__file_lineno}: }",
+			      NULL);
+	rpmlog(error ? RPMLOG_ERR : RPMLOG_WARNING, "%s%s", pfx, emsg);
+	free(pfx);
+    }
+
+    if (error)
+	mb->error = error;
+
+    free(emsg);
+}
+
 /**
  * Pre-print macro expression to be expanded.
  * @param mb		macro expansion state
@@ -394,7 +442,6 @@ static void mbAppend(MacroBuf mb, char c)
     mb->nb--;
 }
 
-#ifdef WITH_LUA
 static void mbAppendStr(MacroBuf mb, const char *str)
 {
     size_t len = strlen(str);
@@ -406,7 +453,14 @@ static void mbAppendStr(MacroBuf mb, const char *str)
     mb->tpos += len;
     mb->nb -= len;
 }
-#endif
+
+static const char * doDnl(MacroBuf mb, const char * se, size_t slen)
+{
+    const char *s = se;
+    while (*s && !iseol(*s))
+	s++;
+    return (*s != '\0') ? s + 1 : s;
+}
 
 /**
  * Expand output of shell command into target buffer.
@@ -445,6 +499,115 @@ exit:
     _free(buf);
 }
 
+
+static unsigned int getncpus(void)
+{
+    unsigned int ncpus = 0;
+#if HAVE_SCHED_GETAFFINITY
+    cpu_set_t set;
+    if (sched_getaffinity (0, sizeof(set), &set) == 0)
+	ncpus = CPU_COUNT(&set);
+#endif
+    /* Fallback to sysconf() if the above isn't supported or didn't work */
+    if (ncpus < 1)
+	ncpus = sysconf(_SC_NPROCESSORS_ONLN);
+    /* If all else fails, there's always the one we're running on... */
+    if (ncpus < 1)
+	ncpus = 1;
+    return ncpus;
+}
+
+#define STR_AND_LEN(_str) (_str), sizeof((_str))-1
+
+/* Names in the table must be in ASCII-code order */
+static struct builtins_s {
+    const char * name;
+    size_t len;
+    macroFunc func;
+    parseFunc parse;
+} const builtinmacros[] = {
+    { STR_AND_LEN("F"),		doFoo,		NULL },
+    { STR_AND_LEN("P"),		doFoo,		NULL },
+    { STR_AND_LEN("S"),		doFoo,		NULL },
+    { STR_AND_LEN("basename"),	doFoo,		NULL },
+    { STR_AND_LEN("define"),	NULL,		doDef },
+    { STR_AND_LEN("dirname"),	doFoo,		NULL },
+    { STR_AND_LEN("dnl"),	NULL,		doDnl },
+    { STR_AND_LEN("dump"), 	NULL,		doDump },
+    { STR_AND_LEN("echo"),	doOutput,	NULL },
+    { STR_AND_LEN("error"),	doOutput,	NULL },
+    { STR_AND_LEN("expand"),	doFoo,		NULL },
+    { STR_AND_LEN("expr"),	doFoo,		NULL },
+    { STR_AND_LEN("getconfdir"),doFoo,		NULL },
+    { STR_AND_LEN("getenv"),	doFoo,		NULL },
+    { STR_AND_LEN("getncpus"),	doFoo,		NULL },
+    { STR_AND_LEN("global"),	NULL,		doGlobal },
+    { STR_AND_LEN("load"),	doLoad,		NULL },
+    { STR_AND_LEN("lua"),	doLua,		NULL },
+    { STR_AND_LEN("quote"),	doFoo,		NULL },
+    { STR_AND_LEN("shrink"),	doFoo,		NULL },
+    { STR_AND_LEN("suffix"),	doFoo,		NULL },
+    { STR_AND_LEN("trace"),	doTrace,	NULL },
+    { STR_AND_LEN("u2p"),	doFoo,		NULL },
+    { STR_AND_LEN("uncompress"),doFoo,		NULL },
+    { STR_AND_LEN("undefine"),	NULL,		doUndefine },
+    { STR_AND_LEN("url2path"),	doFoo,		NULL },
+    { STR_AND_LEN("verbose"),	doFoo,		NULL },
+    { STR_AND_LEN("warn"),	doOutput,	NULL },
+};
+static const size_t numbuiltins = sizeof(builtinmacros)/sizeof(*builtinmacros);
+
+static int namecmp(const void *name1, const void *name2)
+{
+    struct builtins_s *n1 = (struct builtins_s *)name1;
+    struct builtins_s *n2 = (struct builtins_s *)name2;
+
+    int rc = strncmp(n1->name, n2->name, n1->len);
+    if (rc == 0)
+	rc = n1->len - n2->len;
+    return rc;
+}
+
+/**
+ * Return a pointer to the built-in macro with the given name
+ * @param name		macro name
+ * @param nlen		name length
+ * @return		pointer to the built-in macro or NULL if not found
+ */
+static const struct builtins_s* lookupBuiltin(const char *name, size_t nlen)
+{
+    struct builtins_s macro = {
+	.name = name,
+	.len = nlen,
+    };
+
+    return bsearch(&macro, builtinmacros, numbuiltins, sizeof(*builtinmacros),
+		    namecmp);
+}
+
+static const int
+validName(MacroBuf mb, const char *name, size_t namelen, const char *action)
+{
+    int rc = 0;
+    int c;
+
+    /* Names must start with alphabetic or _ and be at least 3 chars */
+    if (!((c = *name) && (risalpha(c) || c == '_') && (namelen) > 2)) {
+	mbErr(mb, 1, _("Macro %%%s has illegal name (%s)\n"), name, action);
+	goto exit;
+    }
+
+    if (lookupBuiltin(name, namelen)) {
+	mbErr(mb, 1, _("Macro %%%s is a built-in (%s)\n"), name, action);
+	goto exit;
+    }
+
+    rc = 1;
+
+exit:
+    return rc;
+}
+
 /**
  * Parse (and execute) new macro definition.
  * @param mb		macro expansion state
@@ -480,7 +643,7 @@ doDefine(MacroBuf mb, const char * se, size_t slen, int level, int expandbody)
 	    COPYOPTS(oe, s, oc);
 	    s++;	/* skip ) */
 	} else {
-	    rpmlog(RPMLOG_ERR, _("Macro %%%s has unterminated opts\n"), n);
+	    mbErr(mb, 1, _("Macro %%%s has unterminated opts\n"), n);
 	    goto exit;
 	}
     }
@@ -491,8 +654,7 @@ doDefine(MacroBuf mb, const char * se, size_t slen, int level, int expandbody)
     SKIPBLANK(s, c);
     if (c == '{') {	/* XXX permit silent {...} grouping */
 	if ((se = matchchar(s, c, '}')) == NULL) {
-	    rpmlog(RPMLOG_ERR,
-		_("Macro %%%s has unterminated body\n"), n);
+	    mbErr(mb, 1, _("Macro %%%s has unterminated body\n"), n);
 	    se = s;	/* XXX W2DO? */
 	    goto exit;
 	}
@@ -529,8 +691,7 @@ doDefine(MacroBuf mb, const char * se, size_t slen, int level, int expandbody)
 	*be = '\0';
 
 	if (bc || pc) {
-	    rpmlog(RPMLOG_ERR,
-		_("Macro %%%s has unterminated body\n"), n);
+	    mbErr(mb, 1, _("Macro %%%s has unterminated body\n"), n);
 	    se = s;	/* XXX W2DO? */
 	    goto exit;
 	}
@@ -546,24 +707,20 @@ doDefine(MacroBuf mb, const char * se, size_t slen, int level, int expandbody)
 	s++;
     se = s;
 
-    /* Names must start with alphabetic or _ and be at least 3 chars */
-    if (!((c = *n) && (risalpha(c) || c == '_') && (ne - n) > 2)) {
-	rpmlog(RPMLOG_ERR, _("Macro %%%s has illegal name (%s)\n"),
-		n, expandbody ? "%global": "%define");
+    if (!validName(mb, n, ne - n, expandbody ? "%global": "%define"))
 	goto exit;
-    }
 
     if ((be - b) < 1) {
-	rpmlog(RPMLOG_ERR, _("Macro %%%s has empty body\n"), n);
+	mbErr(mb, 1, _("Macro %%%s has empty body\n"), n);
 	goto exit;
     }
 
     if (!isblank(*sbody) && !(*sbody == '\\' && iseol(sbody[1])))
-	rpmlog(RPMLOG_WARNING, _("Macro %%%s needs whitespace before body\n"), n);
+	mbErr(mb, 0, _("Macro %%%s needs whitespace before body\n"), n);
 
     if (expandbody) {
 	if (expandThis(mb, b, 0, &ebody)) {
-	    rpmlog(RPMLOG_ERR, _("Macro %%%s failed to expand\n"), n);
+	    mbErr(mb, 1, _("Macro %%%s failed to expand\n"), n);
 	    goto exit;
 	}
 	b = ebody;
@@ -602,9 +759,7 @@ doUndefine(MacroBuf mb, const char * se, size_t slen)
 	s++;
     se = s;
 
-    /* Names must start with alphabetic or _ and be at least 3 chars */
-    if (!((c = *n) && (risalpha(c) || c == '_') && (ne - n) > 2)) {
-	rpmlog(RPMLOG_ERR, _("Macro %%%s has illegal name (%%undefine)\n"), n);
+    if (!validName(mb, n, ne - n, "%undefine")) {
 	mb->error = 1;
 	goto exit;
     }
@@ -616,6 +771,25 @@ exit:
     return se;
 }
 
+static const char * doDef(MacroBuf mb, const char * se, size_t slen)
+{
+    return doDefine(mb, se, slen, mb->level, 0);
+}
+
+static const char * doGlobal(MacroBuf mb, const char * se, size_t slen)
+{
+    return doDefine(mb, se, slen, RMIL_GLOBAL, 1);
+}
+
+static const char * doDump(MacroBuf mb, const char * se, size_t slen)
+{
+    rpmDumpMacroTable(mb->mc, NULL);
+    while (iseol(*se))
+	se++;
+    return se;
+}
+
+
 /**
  * Free parsed arguments for parameterized macro.
  * @param mb		macro expansion state
@@ -633,8 +807,7 @@ freeArgs(MacroBuf mb)
 	    continue;
 	/* Warn on defined but unused non-automatic, scoped macros */
 	if (!(me->flags & (ME_AUTO|ME_USED))) {
-	    rpmlog(RPMLOG_WARNING,
-			_("Macro %%%s defined but not used within scope\n"),
+	    mbErr(mb, 0, _("Macro %%%s defined but not used within scope\n"),
 			me->name);
 	    /* Only whine once */
 	    me->flags |= ME_USED;
@@ -758,9 +931,8 @@ grabArgs(MacroBuf mb, const rpmMacroEntry me, const char * se,
     {
 	char *name = NULL, *body = NULL;
 	if (c == '?' || strchr(opts, c) == NULL) {
-	    rpmlog(RPMLOG_ERR, _("Unknown option %c in %s(%s)\n"),
+	    mbErr(mb, 1, _("Unknown option %c in %s(%s)\n"),
 			(char)optopt, me->name, opts);
-	    mb->error = 1;
 	    goto exit;
 	}
 
@@ -808,24 +980,25 @@ exit:
     return cont;
 }
 
-/**
- * Perform macro message output
- * @param mb		macro expansion state
- * @param waserror	use rpmlog()?
- * @param msg		message to output
- * @param msglen	no. of bytes in message
- */
-static void
-doOutput(MacroBuf mb, int loglevel, const char * msg, size_t msglen)
+static void doOutput(MacroBuf mb, int chkexist, int negate, const char * f, size_t fn, const char * g, size_t gn)
 {
     char *buf = NULL;
+    int loglevel = RPMLOG_NOTICE; /* assume echo */
+    if (STREQ("error", f, fn)) {
+	loglevel = RPMLOG_ERR;
+	mb->error = 1;
+    } else if (STREQ("warn", f, fn)) {
+	loglevel = RPMLOG_WARNING;
+    }
+    if (gn == 0)
+	g = "";
 
-    (void) expandThis(mb, msg, msglen, &buf);
+    (void) expandThis(mb, g, gn, &buf);
     rpmlog(loglevel, "%s\n", buf);
     _free(buf);
 }
 
-static void doLua(MacroBuf mb, const char * f, size_t fn, const char * g, size_t gn)
+static void doLua(MacroBuf mb, int chkexist, int negate, const char * f, size_t fn, const char * g, size_t gn)
 {
 #ifdef WITH_LUA
     rpmlua lua = NULL; /* Global state. */
@@ -852,14 +1025,14 @@ static void doLua(MacroBuf mb, const char * f, size_t fn, const char * g, size_t
     }
     free(scriptbuf);
 #else
-    rpmlog(RPMLOG_ERR, _("<lua> scriptlet support not built in\n"));
-    mb->error = 1;
+    mbErr(mb, 1, _("<lua> scriptlet support not built in\n"));
 #endif
 }
 
 /**
  * Execute macro primitives.
  * @param mb		macro expansion state
+ * @param chkexist	unused
  * @param negate	should logic be inverted?
  * @param f		beginning of field f
  * @param fn		length of field f
@@ -867,7 +1040,7 @@ static void doLua(MacroBuf mb, const char * f, size_t fn, const char * g, size_t
  * @param gn		length of field g
  */
 static void
-doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
+doFoo(MacroBuf mb, int chkexist, int negate, const char * f, size_t fn,
 		const char * g, size_t gn)
 {
     char *buf = NULL;
@@ -875,6 +1048,7 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
     int c;
     int verbose = (rpmIsVerbose() != 0);
     int expand = (g != NULL && gn > 0);
+    int expandagain = 1;
 
     /* Don't expand %{verbose:...} argument on false condition */
     if (STREQ("verbose", f, fn) && (verbose == negate))
@@ -927,6 +1101,15 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	    b++;
     } else if (STREQ("expand", f, fn) || STREQ("verbose", f, fn)) {
 	b = buf;
+    } else if (STREQ("expr", f, fn)) {
+	char *expr = rpmExprStr(buf);
+	if (expr) {
+	    free(buf);
+	    b = buf = expr;
+	    expandagain = 0;
+	} else {
+	    mb->error = 1;
+	}
     } else if (STREQ("url2path", f, fn) || STREQ("u2p", f, fn)) {
 	(void)urlPath(buf, (const char **)&b);
 	if (*b == '\0') b = "/";
@@ -937,7 +1120,8 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	for (be = b; (c = *be) && !isblank(c);)
 	    be++;
 	*be++ = '\0';
-	(void) rpmFileIsCompressed(b, &compressed);
+	if (rpmFileIsCompressed(b, &compressed))
+	    mb->error = 1;
 	switch (compressed) {
 	default:
 	case COMPRESSED_NOT:
@@ -975,6 +1159,9 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
     } else if (STREQ("getconfdir", f, fn)) {
 	sprintf(buf, "%s", rpmConfigDir());
 	b = buf;
+    } else if (STREQ("getncpus", f, fn)) {
+	sprintf(buf, "%u", getncpus());
+	b = buf;
     } else if (STREQ("S", f, fn)) {
 	for (b = buf; (c = *b) && risdigit(c);)
 	    b++;
@@ -997,11 +1184,52 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
     }
 
     if (b) {
-	(void) expandMacro(mb, b, 0);
+	if (expandagain) {
+	    (void) expandMacro(mb, b, 0);
+	} else {
+	    mbAppendStr(mb, b);
+	}
     }
     free(buf);
 }
 
+static void doLoad(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn)
+{
+    char *arg = NULL;
+    if (g && gn > 0 && expandThis(mb, g, gn, &arg) == 0) {
+	/* Print failure iff %{load:...} or %{!?load:...} */
+	if (loadMacroFile(mb->mc, arg) && chkexist == negate) {
+	    mbErr(mb, 1, _("failed to load macro file %s\n"), arg);
+	}
+    }
+    free(arg);
+}
+
+static void doTrace(MacroBuf mb, int chkexist, int negate,
+		    const char * f, size_t fn, const char * g, size_t gn)
+{
+    mb->expand_trace = mb->macro_trace = (negate ? 0 : mb->depth);
+    if (mb->depth == 1) {
+	print_macro_trace = mb->macro_trace;
+	print_expand_trace = mb->expand_trace;
+    }
+}
+
+static const char *setNegateAndCheck(const char *str, int *pnegate, int *pchkexist) {
+
+    *pnegate = 0;
+    *pchkexist = 0;
+    while ((*str == '?') || (*str == '!')) {
+	if (*str == '!')
+	    *pnegate = !*pnegate;
+	else
+	    (*pchkexist)++;
+	str++;
+    }
+    return str;
+}
+
 /**
  * The main macro recursion loop.
  * @param mb		macro expansion state
@@ -1051,15 +1279,15 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
     store_expand_trace = mb->expand_trace;
 
     if (++mb->depth > max_macro_depth) {
-	rpmlog(RPMLOG_ERR,
+	mbErr(mb, 1,
 		_("Too many levels of recursion in macro expansion. It is likely caused by recursive macro declaration.\n"));
 	mb->depth--;
 	mb->expand_trace = 1;
-	mb->error = 1;
 	goto exit;
     }
 
     while (mb->error == 0 && (c = *s) != '\0') {
+	const struct builtins_s* builtin = NULL;
 	s++;
 	/* Copy text until next macro */
 	switch (c) {
@@ -1082,21 +1310,10 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	g = ge = NULL;
 	if (mb->depth > 1)	/* XXX full expansion for outermost level */
 	    tpos = mb->tpos;	/* save expansion pointer for printExpand */
-	negate = 0;
 	lastc = NULL;
-	chkexist = 0;
 	switch ((c = *s)) {
 	default:		/* %name substitution */
-	    while (*s != '\0' && strchr("!?", *s) != NULL) {
-		switch (*s++) {
-		    case '!':
-			negate = ((negate + 1) % 2);
-			break;
-		    case '?':
-			chkexist++;
-			break;
-		}
-	    }
+	    s = setNegateAndCheck(s, &negate, &chkexist);
 	    f = se = s;
 	    if (*se == '-')
 		se++;
@@ -1122,8 +1339,7 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	    break;
 	case '(':		/* %(...) shell escape */
 	    if ((se = matchchar(s, c, ')')) == NULL) {
-		rpmlog(RPMLOG_ERR, _("Unterminated %c: %s\n"), (char)c, s);
-		mb->error = 1;
+		mbErr(mb, 1, _("Unterminated %c: %s\n"), (char)c, s);
 		continue;
 	    }
 	    if (mb->macro_trace)
@@ -1138,22 +1354,12 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	    break;
 	case '{':		/* %{...}/%{...:...} substitution */
 	    if ((se = matchchar(s, c, '}')) == NULL) {
-		rpmlog(RPMLOG_ERR, _("Unterminated %c: %s\n"), (char)c, s);
-		mb->error = 1;
+		mbErr(mb, 1, _("Unterminated %c: %s\n"), (char)c, s);
 		continue;
 	    }
 	    f = s+1;/* skip { */
 	    se++;	/* skip } */
-	    while (strchr("!?", *f) != NULL) {
-		switch (*f++) {
-		case '!':
-		    negate = ((negate + 1) % 2);
-		    break;
-		case '?':
-		    chkexist++;
-		    break;
-		}
-	    }
+	    f = setNegateAndCheck(f, &negate, &chkexist);
 	    for (fe = f; (c = *fe) && !strchr(" :}", c);)
 		fe++;
 	    switch (c) {
@@ -1178,7 +1384,7 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	    c = '%';	/* XXX only need to save % */
 	    mbAppend(mb, c);
 #if 0
-	    rpmlog(RPMLOG_ERR,
+	    mbErr(mb, 1,
 		    _("A %% is followed by an unparseable macro\n"));
 #endif
 	    s = se;
@@ -1189,95 +1395,13 @@ expandMacro(MacroBuf mb, const char *src, size_t slen)
 	    printMacro(mb, s, se);
 
 	/* Expand builtin macros */
-	if (STREQ("load", f, fn)) {
-	    char *arg = NULL;
-	    if (g && gn > 0 && expandThis(mb, g, gn, &arg) == 0) {
-		/* Print failure iff %{load:...} or %{!?load:...} */
-		if (loadMacroFile(mb->mc, arg) && chkexist == negate) {
-		    rpmlog(RPMLOG_ERR, _("failed to load macro file %s"), arg);
-		    mb->error = 1;
-		}
+	if ((builtin = lookupBuiltin(f, fn))) {
+	    if (builtin->parse) {
+		s = builtin->parse(mb, se, slen - (se - s));
+	    } else {
+		builtin->func(mb, chkexist, negate, f, fn, g, gn);
+		s = se;
 	    }
-	    free(arg);
-	    s = se;
-	    continue;
-	}
-	if (STREQ("global", f, fn)) {
-	    s = doDefine(mb, se, slen - (se - s), RMIL_GLOBAL, 1);
-	    continue;
-	}
-	if (STREQ("define", f, fn)) {
-	    s = doDefine(mb, se, slen - (se - s), mb->level, 0);
-	    continue;
-	}
-	if (STREQ("undefine", f, fn)) {
-	    s = doUndefine(mb, se, slen - (se - s));
-	    continue;
-	}
-
-	if (STREQ("echo", f, fn) ||
-		STREQ("warn", f, fn) ||
-		STREQ("error", f, fn)) {
-	    int loglevel = RPMLOG_NOTICE; /* assume echo */
-	    if (STREQ("error", f, fn)) {
-		loglevel = RPMLOG_ERR;
-		mb->error = 1;
-	    } else if (STREQ("warn", f, fn)) {
-		loglevel = RPMLOG_WARNING;
-	    }
-	    if (g != NULL && g < ge)
-		doOutput(mb, loglevel, g, gn);
-	    else
-		doOutput(mb, loglevel, "", 0);
-	    s = se;
-	    continue;
-	}
-
-	if (STREQ("trace", f, fn)) {
-	    /* XXX TODO restore expand_trace/macro_trace to 0 on return */
-	    mb->expand_trace = mb->macro_trace = (negate ? 0 : mb->depth);
-	    if (mb->depth == 1) {
-		print_macro_trace = mb->macro_trace;
-		print_expand_trace = mb->expand_trace;
-	    }
-	    s = se;
-	    continue;
-	}
-
-	if (STREQ("dump", f, fn)) {
-	    rpmDumpMacroTable(mb->mc, NULL);
-	    while (iseol(*se))
-		se++;
-	    s = se;
-	    continue;
-	}
-
-	if (STREQ("lua", f, fn)) {
-	    doLua(mb, f, fn, g, gn);
-	    s = se;
-	    continue;
-	}
-
-	/* XXX necessary but clunky */
-	if (STREQ("basename", f, fn) ||
-	    STREQ("dirname", f, fn) ||
-	    STREQ("shrink", f, fn) ||
-	    STREQ("suffix", f, fn) ||
-	    STREQ("quote", f, fn) ||
-	    STREQ("expand", f, fn) ||
-	    STREQ("verbose", f, fn) ||
-	    STREQ("uncompress", f, fn) ||
-	    STREQ("url2path", f, fn) ||
-	    STREQ("u2p", f, fn) ||
-	    STREQ("getenv", f, fn) ||
-	    STREQ("getconfdir", f, fn) ||
-	    STREQ("S", f, fn) ||
-	    STREQ("P", f, fn) ||
-	    STREQ("F", f, fn))
-	{
-	    /* FIX: verbose may be set */
-	    doFoo(mb, negate, f, fn, g, gn);
-	    s = se;
 	    continue;
 	}
 
@@ -1504,24 +1628,38 @@ static int loadMacroFile(rpmMacroContext mc, const char * fn)
     size_t blen = MACROBUFSIZ;
     char *buf = xmalloc(blen);
     int rc = -1;
+    int nfailed = 0;
+    int lineno = 0;
+    int nlines = 0;
 
     if (fd == NULL)
 	goto exit;
 
-    buf[0] = '\0';
-    while (rdcl(buf, blen, fd) != NULL) {
-	char c, *n;
+    pushMacro(mc, "__file_name", NULL, fn, RMIL_MACROFILES, ME_NONE);
 
+    buf[0] = '\0';
+    while ((nlines = rdcl(buf, blen, fd)) > 0) {
+	char c, *n;
+	char lnobuf[16];
+
+	lineno += nlines;
 	n = buf;
 	SKIPBLANK(n, c);
 
 	if (c != '%')
 		continue;
 	n++;	/* skip % */
-	rc = defineMacro(mc, n, RMIL_MACROFILES);
-    }
 
-    rc = fclose(fd);
+	snprintf(lnobuf, sizeof(lnobuf), "%d", lineno);
+	pushMacro(mc, "__file_lineno", NULL, lnobuf, RMIL_MACROFILES, ME_NONE);
+	if (defineMacro(mc, n, RMIL_MACROFILES))
+	    nfailed++;
+	popMacro(mc, "__file_lineno");
+    }
+    fclose(fd);
+    popMacro(mc, "__file_name");
+
+    rc = (nfailed > 0);
 
 exit:
     _free(buf);
