@@ -41,6 +41,7 @@
 #include <gelf.h>
 #include <dwarf.h>
 
+
 /* Unfortunately strtab manipulation functions were only officially added
    to elfutils libdw in 0.167.  Before that there were internal unsupported
    ebl variants.  While libebl.h isn't supported we'll try to use it anyway
@@ -88,6 +89,8 @@ int do_build_id = 0;
 int no_recompute_build_id = 0;
 char *build_id_seed = NULL;
 
+int show_version = 0;
+
 /* We go over the debug sections in two phases. In phase zero we keep
    track of any needed changes and collect strings, indexes and
    sizes. In phase one we do the actual replacements updating the
@@ -399,13 +402,18 @@ dwarf2_write_be32 (unsigned char *p, uint32_t v)
    relend). Might just update the addend. So relocations need to be
    updated at the end.  */
 
+static bool rel_updated;
+
 #define do_write_32_relocated(ptr,val) ({ \
   if (relptr && relptr < relend && relptr->ptr == ptr)	\
     {							\
       if (reltype == SHT_REL)				\
 	do_write_32 (ptr, val - relptr->addend);	\
       else						\
-	relptr->addend = val;				\
+	{						\
+	  relptr->addend = val;				\
+	  rel_updated = true;				\
+	}						\
     }							\
   else							\
     do_write_32 (ptr,val);				\
@@ -416,14 +424,19 @@ dwarf2_write_be32 (unsigned char *p, uint32_t v)
   ptr += 4;			       \
 })
 
-static struct
+typedef struct debug_section
   {
     const char *name;
     unsigned char *data;
     Elf_Data *elf_data;
     size_t size;
     int sec, relsec;
-  } debug_sections[] =
+    REL *relbuf;
+    REL *relend;
+    struct debug_section *next; /* Only happens for COMDAT .debug_macro.  */
+  } debug_section;
+
+static debug_section debug_sections[] =
   {
 #define DEBUG_INFO	0
 #define DEBUG_ABBREV	1
@@ -456,6 +469,201 @@ static struct
     { NULL, NULL, NULL, 0, 0, 0 }
   };
 
+static int
+rel_cmp (const void *a, const void *b)
+{
+  REL *rela = (REL *) a, *relb = (REL *) b;
+
+  if (rela->ptr < relb->ptr)
+    return -1;
+
+  if (rela->ptr > relb->ptr)
+    return 1;
+
+  return 0;
+}
+
+/* Returns a malloced REL array, or NULL when there are no relocations
+   for this section.  When there are relocations, will setup relend,
+   as the last REL, and reltype, as SHT_REL or SHT_RELA.  */
+static void
+setup_relbuf (DSO *dso, debug_section *sec, int *reltype)
+{
+  int ndx, maxndx;
+  GElf_Rel rel;
+  GElf_Rela rela;
+  GElf_Sym sym;
+  GElf_Addr base = dso->shdr[sec->sec].sh_addr;
+  Elf_Data *symdata = NULL;
+  int rtype;
+  REL *relbuf;
+  Elf_Scn *scn;
+  Elf_Data *data;
+  int i = sec->relsec;
+
+  /* No relocations, or did we do this already? */
+  if (i == 0 || sec->relbuf != NULL)
+    {
+      relptr = sec->relbuf;
+      relend = sec->relend;
+      return;
+    }
+
+  scn = dso->scn[i];
+  data = elf_getdata (scn, NULL);
+  assert (data != NULL && data->d_buf != NULL);
+  assert (elf_getdata (scn, data) == NULL);
+  assert (data->d_off == 0);
+  assert (data->d_size == dso->shdr[i].sh_size);
+  maxndx = dso->shdr[i].sh_size / dso->shdr[i].sh_entsize;
+  relbuf = malloc (maxndx * sizeof (REL));
+  *reltype = dso->shdr[i].sh_type;
+  if (relbuf == NULL)
+    error (1, errno, "%s: Could not allocate memory", dso->filename);
+
+  symdata = elf_getdata (dso->scn[dso->shdr[i].sh_link], NULL);
+  assert (symdata != NULL && symdata->d_buf != NULL);
+  assert (elf_getdata (dso->scn[dso->shdr[i].sh_link], symdata) == NULL);
+  assert (symdata->d_off == 0);
+  assert (symdata->d_size == dso->shdr[dso->shdr[i].sh_link].sh_size);
+
+  for (ndx = 0, relend = relbuf; ndx < maxndx; ++ndx)
+    {
+      if (dso->shdr[i].sh_type == SHT_REL)
+	{
+	  gelf_getrel (data, ndx, &rel);
+	  rela.r_offset = rel.r_offset;
+	  rela.r_info = rel.r_info;
+	  rela.r_addend = 0;
+	}
+      else
+	gelf_getrela (data, ndx, &rela);
+      gelf_getsym (symdata, ELF64_R_SYM (rela.r_info), &sym);
+      /* Relocations against section symbols are uninteresting in REL.  */
+      if (dso->shdr[i].sh_type == SHT_REL && sym.st_value == 0)
+	continue;
+      /* Only consider relocations against .debug_str, .debug_line
+	 and .debug_abbrev.  */
+      if (sym.st_shndx != debug_sections[DEBUG_STR].sec
+	  && sym.st_shndx != debug_sections[DEBUG_LINE].sec
+	  && sym.st_shndx != debug_sections[DEBUG_ABBREV].sec)
+	continue;
+      rela.r_addend += sym.st_value;
+      rtype = ELF64_R_TYPE (rela.r_info);
+      switch (dso->ehdr.e_machine)
+	{
+	case EM_SPARC:
+	case EM_SPARC32PLUS:
+	case EM_SPARCV9:
+	  if (rtype != R_SPARC_32 && rtype != R_SPARC_UA32)
+	    goto fail;
+	  break;
+	case EM_386:
+	  if (rtype != R_386_32)
+	    goto fail;
+	  break;
+	case EM_PPC:
+	case EM_PPC64:
+	  if (rtype != R_PPC_ADDR32 && rtype != R_PPC_UADDR32)
+	    goto fail;
+	  break;
+	case EM_S390:
+	  if (rtype != R_390_32)
+	    goto fail;
+	  break;
+	case EM_IA_64:
+	  if (rtype != R_IA64_SECREL32LSB)
+	    goto fail;
+	  break;
+	case EM_X86_64:
+	  if (rtype != R_X86_64_32)
+	    goto fail;
+	  break;
+	case EM_ALPHA:
+	  if (rtype != R_ALPHA_REFLONG)
+	    goto fail;
+	  break;
+#if defined(EM_AARCH64) && defined(R_AARCH64_ABS32)
+	case EM_AARCH64:
+	  if (rtype != R_AARCH64_ABS32)
+	    goto fail;
+	  break;
+#endif
+	case EM_68K:
+	  if (rtype != R_68K_32)
+	    goto fail;
+	  break;
+#if defined(EM_RISCV) && defined(R_RISCV_32)
+	case EM_RISCV:
+	  if (rtype != R_RISCV_32)
+	    goto fail;
+	  break;
+#endif
+	default:
+	fail:
+	  error (1, 0, "%s: Unhandled relocation %d in %s section",
+		 dso->filename, rtype, sec->name);
+	}
+      relend->ptr = sec->data
+	+ (rela.r_offset - base);
+      relend->addend = rela.r_addend;
+      relend->ndx = ndx;
+      ++(relend);
+    }
+  if (relbuf == relend)
+    {
+      free (relbuf);
+      relbuf = NULL;
+      relend = NULL;
+    }
+  else
+    qsort (relbuf, relend - relbuf, sizeof (REL), rel_cmp);
+
+  sec->relbuf = relbuf;
+  sec->relend = relend;
+  relptr = relbuf;
+}
+
+/* Updates SHT_RELA section associated with the given section based on
+   the relbuf data. The relbuf data is freed at the end.  */
+static void
+update_rela_data (DSO *dso, struct debug_section *sec)
+{
+  Elf_Data *symdata;
+  int relsec_ndx = sec->relsec;
+  Elf_Data *data = elf_getdata (dso->scn[relsec_ndx], NULL);
+  symdata = elf_getdata (dso->scn[dso->shdr[relsec_ndx].sh_link],
+			 NULL);
+
+  relptr = sec->relbuf;
+  relend = sec->relend;
+  while (relptr < relend)
+    {
+      GElf_Sym sym;
+      GElf_Rela rela;
+      int ndx = relptr->ndx;
+
+      if (gelf_getrela (data, ndx, &rela) == NULL)
+	error (1, 0, "Couldn't get relocation: %s",
+	       elf_errmsg (-1));
+
+      if (gelf_getsym (symdata, GELF_R_SYM (rela.r_info),
+		       &sym) == NULL)
+	error (1, 0, "Couldn't get symbol: %s", elf_errmsg (-1));
+
+      rela.r_addend = relptr->addend - sym.st_value;
+
+      if (gelf_update_rela (data, ndx, &rela) == 0)
+	error (1, 0, "Couldn't update relocations: %s",
+	       elf_errmsg (-1));
+
+      ++relptr;
+    }
+  elf_flagdata (data, ELF_C_SET, ELF_F_DIRTY);
+
+  free (sec->relbuf);
+}
+
 struct abbrev_attr
   {
     unsigned int attr;
@@ -820,6 +1028,9 @@ record_file_string_entry_idx (struct strings *strings, size_t old_idx)
   struct stridxentry *entry = string_find_new_entry (strings, old_idx);
   if (entry != NULL)
     {
+      if (old_idx >= debug_sections[DEBUG_STR].size)
+	error (1, 0, "Bad string pointer index %zd", old_idx);
+
       Strent *strent;
       const char *old_str = (char *)debug_sections[DEBUG_STR].data + old_idx;
       const char *file = skip_dir_prefix (old_str, base_dir);
@@ -870,6 +1081,9 @@ record_existing_string_entry_idx (struct strings *strings, size_t old_idx)
   struct stridxentry *entry = string_find_new_entry (strings, old_idx);
   if (entry != NULL)
     {
+      if (old_idx >= debug_sections[DEBUG_STR].size)
+	error (1, 0, "Bad string pointer index %zd", old_idx);
+
       const char *str = (char *)debug_sections[DEBUG_STR].data + old_idx;
       Strent *strent = strtab_add_len (strings->str_tab,
 				       str, strlen (str) + 1);
@@ -963,6 +1177,7 @@ get_line_table (DSO *dso, size_t off, struct line_table **table)
   *table = NULL;
 
   t->old_idx = off;
+  t->new_idx = off;
   t->size_diff = 0;
   t->replace_dirs = false;
   t->replace_files = false;
@@ -1533,10 +1748,18 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 		{
 		  const char *dir;
 		  size_t idx = do_read_32_relocated (ptr);
-		  dir = (char *) debug_sections[DEBUG_STR].data + idx;
+		  /* In phase zero we collect the comp_dir.  */
+		  if (phase == 0)
+		    {
+		      if (idx >= debug_sections[DEBUG_STR].size)
+			error (1, 0,
+			       "%s: Bad string pointer index %zd for comp_dir",
+			       dso->filename, idx);
+		      dir = (char *) debug_sections[DEBUG_STR].data + idx;
 
-		  free (comp_dir);
-		  comp_dir = strdup (dir);
+		      free (comp_dir);
+		      comp_dir = strdup (dir);
+		    }
 
 		  if (dest_dir != NULL && phase == 0)
 		    {
@@ -1556,21 +1779,29 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 		 unit. If starting with / it is a full path name.
 		 Note that we don't handle DW_FORM_string in this
 		 case.  */
-	      char *name;
 	      size_t idx = do_read_32_relocated (ptr);
-	      name = (char *) debug_sections[DEBUG_STR].data + idx;
-	      if (*name == '/' && comp_dir == NULL)
-		{
-		  char *enddir = strrchr (name, '/');
 
-		  if (enddir != name)
+	      /* In phase zero we will look for a comp_dir to use.  */
+	      if (phase == 0)
+		{
+		  if (idx >= debug_sections[DEBUG_STR].size)
+		    error (1, 0,
+			   "%s: Bad string pointer index %zd for unit name",
+			   dso->filename, idx);
+		  char *name = (char *) debug_sections[DEBUG_STR].data + idx;
+		  if (*name == '/' && comp_dir == NULL)
 		    {
-		      comp_dir = malloc (enddir - name + 1);
-		      memcpy (comp_dir, name, enddir - name);
-		      comp_dir [enddir - name] = '\0';
+		      char *enddir = strrchr (name, '/');
+
+		      if (enddir != name)
+			{
+			  comp_dir = malloc (enddir - name + 1);
+			  memcpy (comp_dir, name, enddir - name);
+			  comp_dir [enddir - name] = '\0';
+			}
+		      else
+			comp_dir = strdup ("/");
 		    }
-		  else
-		    comp_dir = strdup ("/");
 		}
 
 	      /* First pass (0) records the new name to be
@@ -1719,20 +1950,6 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
   return ptr;
 }
 
-static int
-rel_cmp (const void *a, const void *b)
-{
-  REL *rela = (REL *) a, *relb = (REL *) b;
-
-  if (rela->ptr < relb->ptr)
-    return -1;
-
-  if (rela->ptr > relb->ptr)
-    return 1;
-
-  return 0;
-}
-
 static int
 line_rel_cmp (const void *a, const void *b)
 {
@@ -1775,11 +1992,35 @@ edit_dwarf2 (DSO *dso)
 	    for (j = 0; debug_sections[j].name; ++j)
 	      if (strcmp (name, debug_sections[j].name) == 0)
 	 	{
+		  struct debug_section *debug_sec = &debug_sections[j];
 		  if (debug_sections[j].data)
 		    {
-		      error (0, 0, "%s: Found two copies of %s section",
-			     dso->filename, name);
-		      return 1;
+		      if (j != DEBUG_MACRO)
+			{
+			  error (0, 0, "%s: Found two copies of %s section",
+				 dso->filename, name);
+			  return 1;
+			}
+		      else
+			{
+			  /* In relocatable files .debug_macro might
+			     appear multiple times as COMDAT
+			     section.  */
+			  struct debug_section *sec;
+			  sec = calloc (sizeof (struct debug_section), 1);
+			  if (sec == NULL)
+			    error (1, errno,
+				   "%s: Could not allocate more macro sections",
+				   dso->filename);
+			  sec->name = ".debug_macro";
+
+			  struct debug_section *macro_sec = debug_sec;
+			  while (macro_sec->next != NULL)
+			    macro_sec = macro_sec->next;
+
+			  macro_sec->next = sec;
+			  debug_sec = sec;
+			}
 		    }
 
 		  scn = dso->scn[i];
@@ -1788,10 +2029,10 @@ edit_dwarf2 (DSO *dso)
 		  assert (elf_getdata (scn, data) == NULL);
 		  assert (data->d_off == 0);
 		  assert (data->d_size == dso->shdr[i].sh_size);
-		  debug_sections[j].data = data->d_buf;
-		  debug_sections[j].elf_data = data;
-		  debug_sections[j].size = data->d_size;
-		  debug_sections[j].sec = i;
+		  debug_sec->data = data->d_buf;
+		  debug_sec->elf_data = data;
+		  debug_sec->size = data->d_size;
+		  debug_sec->sec = i;
 		  break;
 		}
 
@@ -1814,7 +2055,26 @@ edit_dwarf2 (DSO *dso)
 			  + (dso->shdr[i].sh_type == SHT_RELA),
 			  debug_sections[j].name) == 0)
 	 	{
-		  debug_sections[j].relsec = i;
+		  if (j == DEBUG_MACRO)
+		    {
+		      /* Pick the correct one.  */
+		      int rel_target = dso->shdr[i].sh_info;
+		      struct debug_section *macro_sec = &debug_sections[j];
+		      while (macro_sec != NULL)
+			{
+			  if (macro_sec->sec == rel_target)
+			    {
+			      macro_sec->relsec = i;
+			      break;
+			    }
+			  macro_sec = macro_sec->next;
+			}
+		      if (macro_sec == NULL)
+			error (0, 1, "No .debug_macro reloc section: %s",
+			       dso->filename);
+		    }
+		  else
+		    debug_sections[j].relsec = i;
 		  break;
 		}
 	  }
@@ -1847,126 +2107,8 @@ edit_dwarf2 (DSO *dso)
       htab_t abbrev;
       struct abbrev_tag tag, *t;
       int phase;
-      REL *relbuf = NULL;
-
-      if (debug_sections[DEBUG_INFO].relsec)
-	{
-	  int ndx, maxndx;
-	  GElf_Rel rel;
-	  GElf_Rela rela;
-	  GElf_Sym sym;
-	  GElf_Addr base = dso->shdr[debug_sections[DEBUG_INFO].sec].sh_addr;
-	  Elf_Data *symdata = NULL;
-	  int rtype;
-
-	  i = debug_sections[DEBUG_INFO].relsec;
-	  scn = dso->scn[i];
-	  data = elf_getdata (scn, NULL);
-	  assert (data != NULL && data->d_buf != NULL);
-	  assert (elf_getdata (scn, data) == NULL);
-	  assert (data->d_off == 0);
-	  assert (data->d_size == dso->shdr[i].sh_size);
-	  maxndx = dso->shdr[i].sh_size / dso->shdr[i].sh_entsize;
-	  relbuf = malloc (maxndx * sizeof (REL));
-	  reltype = dso->shdr[i].sh_type;
-	  if (relbuf == NULL)
-	    error (1, errno, "%s: Could not allocate memory", dso->filename);
-
-	  symdata = elf_getdata (dso->scn[dso->shdr[i].sh_link], NULL);
-	  assert (symdata != NULL && symdata->d_buf != NULL);
-	  assert (elf_getdata (dso->scn[dso->shdr[i].sh_link], symdata)
-		  == NULL);
-	  assert (symdata->d_off == 0);
-	  assert (symdata->d_size
-		  == dso->shdr[dso->shdr[i].sh_link].sh_size);
-
-	  for (ndx = 0, relend = relbuf; ndx < maxndx; ++ndx)
-	    {
-	      if (dso->shdr[i].sh_type == SHT_REL)
-		{
-		  gelf_getrel (data, ndx, &rel);
-		  rela.r_offset = rel.r_offset;
-		  rela.r_info = rel.r_info;
-		  rela.r_addend = 0;
-		}
-	      else
-		gelf_getrela (data, ndx, &rela);
-	      gelf_getsym (symdata, ELF64_R_SYM (rela.r_info), &sym);
-	      /* Relocations against section symbols are uninteresting
-		 in REL.  */
-	      if (dso->shdr[i].sh_type == SHT_REL && sym.st_value == 0)
-		continue;
-	      /* Only consider relocations against .debug_str, .debug_line
-		 and .debug_abbrev.  */
-	      if (sym.st_shndx != debug_sections[DEBUG_STR].sec
-		  && sym.st_shndx != debug_sections[DEBUG_LINE].sec
-		  && sym.st_shndx != debug_sections[DEBUG_ABBREV].sec)
-		continue;
-	      rela.r_addend += sym.st_value;
-	      rtype = ELF64_R_TYPE (rela.r_info);
-	      switch (dso->ehdr.e_machine)
-		{
-		case EM_SPARC:
-		case EM_SPARC32PLUS:
-		case EM_SPARCV9:
-		  if (rtype != R_SPARC_32 && rtype != R_SPARC_UA32)
-		    goto fail;
-		  break;
-		case EM_386:
-		  if (rtype != R_386_32)
-		    goto fail;
-		  break;
-		case EM_PPC:
-		case EM_PPC64:
-		  if (rtype != R_PPC_ADDR32 && rtype != R_PPC_UADDR32)
-		    goto fail;
-		  break;
-		case EM_S390:
-		  if (rtype != R_390_32)
-		    goto fail;
-		  break;
-		case EM_IA_64:
-		  if (rtype != R_IA64_SECREL32LSB)
-		    goto fail;
-		  break;
-		case EM_X86_64:
-		  if (rtype != R_X86_64_32)
-		    goto fail;
-		  break;
-		case EM_ALPHA:
-		  if (rtype != R_ALPHA_REFLONG)
-		    goto fail;
-		  break;
-#if defined(EM_AARCH64) && defined(R_AARCH64_ABS32)
-		case EM_AARCH64:
-		  if (rtype != R_AARCH64_ABS32)
-		    goto fail;
-		  break;
-#endif
-		case EM_68K:
-		  if (rtype != R_68K_32)
-		    goto fail;
-		  break;
-		default:
-		fail:
-		  error (1, 0, "%s: Unhandled relocation %d in .debug_info section",
-			 dso->filename, rtype);
-		}
-	      relend->ptr = debug_sections[DEBUG_INFO].data
-			    + (rela.r_offset - base);
-	      relend->addend = rela.r_addend;
-	      relend->ndx = ndx;
-	      ++relend;
-	    }
-	  if (relbuf == relend)
-	    {
-	      free (relbuf);
-	      relbuf = NULL;
-	      relend = NULL;
-	    }
-	  else
-	    qsort (relbuf, relend - relbuf, sizeof (REL), rel_cmp);
-	}
+      bool info_rel_updated = false;
+      bool macro_rel_updated = false;
 
       for (phase = 0; phase < 2; phase++)
 	{
@@ -1978,7 +2120,8 @@ edit_dwarf2 (DSO *dso)
 	    break;
 
 	  ptr = debug_sections[DEBUG_INFO].data;
-	  relptr = relbuf;
+	  setup_relbuf(dso, &debug_sections[DEBUG_INFO], &reltype);
+	  rel_updated = false;
 	  endsec = ptr + debug_sections[DEBUG_INFO].size;
 	  while (ptr < endsec)
 	    {
@@ -2066,6 +2209,10 @@ edit_dwarf2 (DSO *dso)
 	      htab_delete (abbrev);
 	    }
 
+	  /* Remember whether any .debug_info relocations might need
+	     to be updated. */
+	  info_rel_updated = rel_updated;
+
 	  /* We might have to recalculate/rewrite the debug_line
 	     section.  We need to do that before going into phase one
 	     so we have all new offsets.  We do this separately from
@@ -2154,9 +2301,9 @@ edit_dwarf2 (DSO *dso)
 		      /* Offset (pointing into the line program) moves
 			 from old to new index including the header
 			 size diff. */
-		      r_offset += ((dso->lines.table[lndx].new_idx
-				    - dso->lines.table[lndx].old_idx)
-				   + dso->lines.table[lndx].size_diff);
+		      r_offset += (ssize_t)((dso->lines.table[lndx].new_idx
+					     - dso->lines.table[lndx].old_idx)
+					    + dso->lines.table[lndx].size_diff);
 
 		      if (rtype == SHT_RELA)
 			{
@@ -2179,6 +2326,113 @@ edit_dwarf2 (DSO *dso)
 		}
 	    }
 
+	  /* The .debug_macro section also contains offsets into the
+	     .debug_str section and references to the .debug_line
+	     tables, so we need to update those as well if we update
+	     the strings or the stmts.  */
+	  if ((need_strp_update || need_stmt_update)
+	      && debug_sections[DEBUG_MACRO].data)
+	    {
+	      /* There might be multiple (COMDAT) .debug_macro sections.  */
+	      struct debug_section *macro_sec = &debug_sections[DEBUG_MACRO];
+	      while (macro_sec != NULL)
+		{
+		  setup_relbuf(dso, macro_sec, &reltype);
+		  rel_updated = false;
+
+		  ptr = macro_sec->data;
+		  endsec = ptr + macro_sec->size;
+		  int op = 0, macro_version, macro_flags;
+		  int offset_len = 4, line_offset = 0;
+
+		  while (ptr < endsec)
+		    {
+		      if (!op)
+			{
+			  macro_version = read_16 (ptr);
+			  macro_flags = read_8 (ptr);
+			  if (macro_version < 4 || macro_version > 5)
+			    error (1, 0, "unhandled .debug_macro version: %d",
+				   macro_version);
+			  if ((macro_flags & ~2) != 0)
+			    error (1, 0, "unhandled .debug_macro flags: 0x%x",
+				   macro_flags);
+
+			  offset_len = (macro_flags & 0x01) ? 8 : 4;
+			  line_offset = (macro_flags & 0x02) ? 1 : 0;
+
+			  if (offset_len != 4)
+			    error (0, 1,
+				   "Cannot handle 8 byte macro offsets: %s",
+				   dso->filename);
+
+			  /* Update the line_offset if it is there.  */
+			  if (line_offset)
+			    {
+			      if (phase == 0)
+				ptr += offset_len;
+			      else
+				{
+				  size_t idx, new_idx;
+				  idx = do_read_32_relocated (ptr);
+				  new_idx = find_new_list_offs (&dso->lines,
+								idx);
+				  write_32_relocated (ptr, new_idx);
+				}
+			    }
+			}
+
+		      op = read_8 (ptr);
+		      if (!op)
+			continue;
+		      switch(op)
+			{
+			case DW_MACRO_GNU_define:
+			case DW_MACRO_GNU_undef:
+			  read_uleb128 (ptr);
+			  ptr = ((unsigned char *) strchr ((char *) ptr, '\0')
+				 + 1);
+			  break;
+			case DW_MACRO_GNU_start_file:
+			  read_uleb128 (ptr);
+			  read_uleb128 (ptr);
+			  break;
+			case DW_MACRO_GNU_end_file:
+			  break;
+			case DW_MACRO_GNU_define_indirect:
+			case DW_MACRO_GNU_undef_indirect:
+			  read_uleb128 (ptr);
+			  if (phase == 0)
+			    {
+			      size_t idx = read_32_relocated (ptr);
+			      record_existing_string_entry_idx (&dso->strings,
+								idx);
+			    }
+			  else
+			    {
+			      struct stridxentry *entry;
+			      size_t idx, new_idx;
+			      idx = do_read_32_relocated (ptr);
+			      entry = string_find_entry (&dso->strings, idx);
+			      new_idx = strent_offset (entry->entry);
+			      write_32_relocated (ptr, new_idx);
+			    }
+			  break;
+			case DW_MACRO_GNU_transparent_include:
+			  ptr += offset_len;
+			  break;
+			default:
+			  error (1, 0, "Unhandled DW_MACRO op 0x%x", op);
+			  break;
+			}
+		    }
+
+		  if (rel_updated)
+		    macro_rel_updated = true;
+		  macro_sec = macro_sec->next;
+		}
+	    }
+
 	  /* Same for the debug_str section. Make sure everything is
 	     in place for phase 1 updating of debug_info
 	     references. */
@@ -2208,43 +2462,24 @@ edit_dwarf2 (DSO *dso)
 	 new strp, strings and/or linep offsets.  */
       if (need_strp_update || need_string_replacement || need_stmt_update)
 	dirty_section (DEBUG_INFO);
+      if (need_strp_update || need_stmt_update)
+	dirty_section (DEBUG_MACRO);
+      if (need_stmt_update)
+	dirty_section (DEBUG_LINE);
 
-      /* Update any debug_info relocations addends we might have touched. */
-      if (relbuf != NULL && reltype == SHT_RELA)
+      /* Update any relocations addends we might have touched. */
+      if (info_rel_updated)
+	update_rela_data (dso, &debug_sections[DEBUG_INFO]);
+
+      if (macro_rel_updated)
 	{
-	  Elf_Data *symdata;
-          int relsec_ndx = debug_sections[DEBUG_INFO].relsec;
-          data = elf_getdata (dso->scn[relsec_ndx], NULL);
-	  symdata = elf_getdata (dso->scn[dso->shdr[relsec_ndx].sh_link],
-				 NULL);
-
-	  relptr = relbuf;
-	  while (relptr < relend)
+	  struct debug_section *macro_sec = &debug_sections[DEBUG_MACRO];
+	  while (macro_sec != NULL)
 	    {
-	      GElf_Sym sym;
-	      GElf_Rela rela;
-	      int ndx = relptr->ndx;
-
-	      if (gelf_getrela (data, ndx, &rela) == NULL)
-		error (1, 0, "Couldn't get relocation: %s",
-		       elf_errmsg (-1));
-
-	      if (gelf_getsym (symdata, GELF_R_SYM (rela.r_info),
-			       &sym) == NULL)
-		error (1, 0, "Couldn't get symbol: %s", elf_errmsg (-1));
-
-	      rela.r_addend = relptr->addend - sym.st_value;
-
-	      if (gelf_update_rela (data, ndx, &rela) == 0)
-		error (1, 0, "Couldn't update relocations: %s",
-		       elf_errmsg (-1));
-
-	      ++relptr;
+	      update_rela_data (dso, macro_sec);
+	      macro_sec = macro_sec->next;
 	    }
-	  elf_flagdata (data, ELF_C_SET, ELF_F_DIRTY);
 	}
-
-      free (relbuf);
     }
 
   return 0;
@@ -2263,6 +2498,8 @@ static struct poptOption optionsTable[] = {
       "if recomputing the build ID note use this string as hash seed", NULL },
     { "no-recompute-build-id",  'n', POPT_ARG_NONE, &no_recompute_build_id, 0,
       "do not recompute build ID note even when -i or -s are given", NULL },
+    { "version", '\0', POPT_ARG_NONE, &show_version, 0,
+      "print the debugedit version", NULL },
       POPT_AUTOHELP
     { NULL, 0, 0, NULL, 0, NULL, NULL }
 };
@@ -2499,6 +2736,12 @@ main (int argc, char *argv[])
       exit (1);
     }
 
+  if (show_version)
+    {
+      printf("RPM debugedit %s\n", VERSION);
+      exit(EXIT_SUCCESS);
+    }
+
   args = poptGetArgs (optCon);
   if (args == NULL || args[0] == NULL || args[1] != NULL)
     {
@@ -2768,6 +3011,17 @@ main (int argc, char *argv[])
   destroy_lines (&dso->lines);
   free (dso);
 
+  /* In case there were multiple (COMDAT) .debug_macro sections,
+     free them.  */
+  struct debug_section *macro_sec = &debug_sections[DEBUG_MACRO];
+  macro_sec = macro_sec->next;
+  while (macro_sec != NULL)
+    {
+      struct debug_section *next = macro_sec->next;
+      free (macro_sec);
+      macro_sec = next;
+    }
+
   poptFreeContext (optCon);
 
   return 0;
